/************************************************************************
*                                                                       *
*       The program of making message log                               *
*                                                                       *
************************************************************************/
#include <stdio.h>
#include <string.h>
#include <direct.h>

#define SPACE_CHAR            ' '

#define VOL_LEN                 4
#define GROUP_LEN               4
#define USER_LEN                6
#define MSG_STRING_MAX        100

static FILE *fp = NULL;         /* Log file pointer */

/************************************************************************
*                                                                       *
*   Open the log file                                                   *
*                                                                       *
*       path    : Path Name                                             *
*                                                                       *
************************************************************************/
short logfopen(char *path)
{
    short   retc;

    /*------------------*/
    /* Initial          */
    /*------------------*/
    retc = 0;
    /*------------------*/
    /* Open the file    */
    /*------------------*/
    fp = fopen(path, "wt");
    if (fp == NULL) retc = 1;
    return(retc);
}
/************************************************************************
*                                                                       *
*   Output message to the console and log file                          *
*                                                                       *
*       buf     : The data for output                                   *
*                                                                       *
************************************************************************/
void logfput(char *buf)
{
    printf("%s", buf);
    if (fp != NULL)
        fprintf(fp, "%s", buf);
}
/************************************************************************
*                                                                       *
*   Close the log file                                                  *
*                                                                       *
************************************************************************/
void logfclse(void)
{
    if (fp != NULL)
        fclose(fp);
    fp = NULL;
}
/************************************************************************
*                                                                       *
*   Output the header message                                           *
*                                                                       *
*       vol     : Volume                                                *
*       grp     : Group                                                 *
*       usr     : User                                                  *
*                                                                       *
************************************************************************/
short loghed(char *vol, char *grp, char *usr)
{
    int     ofst;
    int     len;
    char    msgbuf[MSG_STRING_MAX];
    char    msgbuf1[MSG_STRING_MAX];

    /*------------------*/
    /* Initial          */
    /*------------------*/
    memset(msgbuf, '\0', sizeof(msgbuf));
    /*--------------------------------------------------*/
    /* Make the header message (vol, group and user)    */
    /*--------------------------------------------------*/
    ofst = 0;
    len = VOL_LEN;
    memcpy(&msgbuf[ofst], vol, len);
    ofst += len;
    msgbuf[ofst] = SPACE_CHAR;
    ofst++;
    len = GROUP_LEN;
    memcpy(&msgbuf[ofst], grp, len);
    ofst += len;
    msgbuf[ofst] = SPACE_CHAR;
    ofst++;
    len = USER_LEN;
    memcpy(&msgbuf[ofst], usr, len);
    sprintf(msgbuf1, "%s\n", msgbuf);
    /*------------------------------*/
    /* Output vol, group and user   */
    /*------------------------------*/
    logfput(msgbuf1);
    return(0);
}
/************************************************************************
*                                                                       *
*   Output the drawing name and error code                              *
*                                                                       *
*       dwgid   : Drawing Name                                          *
*       errcode : Error Code                                            *
*                                                                       *
************************************************************************/
short logdwgid(char *dwgid, long errcode)
{
    char    msgbuf[MSG_STRING_MAX];
    char    msgbuf1[MSG_STRING_MAX];

    /*--------------*/
    /* Initial      */
    /*--------------*/
    memset(msgbuf, '\0', sizeof(msgbuf));
    /*--------------------------------------*/
    /* Output the drawing name and message  */
    /*--------------------------------------*/
    switch(errcode) {
    case 0:
        strcpy(msgbuf, "Complete");
        break;
    case 1:
        strcpy(msgbuf, "Error    : Other Error");
        break;
    case 5:
        strcpy(msgbuf, "Error    : Drawing Too Big");
        break;
    case 11:
        strcpy(msgbuf, "Error    : Disk Full");
        break;
    case 16:
        strcpy(msgbuf, "Error    : I/O Error");
        break;
    case 20:
        strcpy(msgbuf, "Error    : RO Drawing");
        break;
    case 21:
        strcpy(msgbuf, "Error    : Incompatible Drawing");
        break;
    case 22:
        strcpy(msgbuf, "Error    : Overlay Parent Drawing");
        break;
    case 23:
        strcpy(msgbuf, "Error    : PW Drawing");
        break;
    case 24:
        strcpy(msgbuf, "Error    : Bad Drawing");
        break;
    case 25:
        strcpy(msgbuf, "Error    : Volume Not Supported");
        break;
    case 26:
        strcpy(msgbuf, "Error    : Volume Not Exits");
        break;
    case 27:
        strcpy(msgbuf, "Error    : Already Renewed");
        break;
    case 31:
        strcpy(msgbuf, "Error    : Same Drawing Has Opened");
        break;
    case 37:
        strcpy(msgbuf, "Error    : SC Drawing");
        break;
    case 72:
        strcpy(msgbuf, "Error    : Group, User Not Exit in Privilege");
        break;
    case 73:
        strcpy(msgbuf, "Error    : Permission Denied");
        break;
    case 100:
        strcpy(msgbuf, "Error    : Drawing Number Exceeded");
        break;
    case 200:
        strcpy(msgbuf, "Error    : Out of Memory");
        break;
    case 400:
        strcpy(msgbuf, "Error    : No Active Member");
        break;
    case 1000:
        strcpy(msgbuf, "Complete : No Change");
        break;
    case 1002:
        strcpy(msgbuf, "Error    : LK Drawing");
        break;
    default:
        strcpy(msgbuf, "Error    : Other Error");
        break;
    } /* end of switch */
    sprintf(msgbuf1, "    %s    %s\n", dwgid, msgbuf);
    logfput(msgbuf1);
    return(0);
}
/************************************************************************
*                                                                       *
*   Output the parameter error message                                  *
*                                                                       *
*       errcode : Error Code                                            *
*                                                                       *
************************************************************************/
short logmsg(long errcode)
{
    char    msgbuf[MSG_STRING_MAX];

    /*--------------*/
    /* Initial      */
    /*--------------*/
    memset(msgbuf, '\0', sizeof(msgbuf));
    /*--------------------------------------*/
    /* Output the parameter error message   */
    /*--------------------------------------*/
    switch(errcode) {
    case 0:
        strcpy(msgbuf, "Complete\n");
        logfput(msgbuf);
        break;
    case 1:
        strcpy(msgbuf, "\nParameter input error\n");
        logfput(msgbuf);
        strcpy(msgbuf, "\ne.g.   ( )-Abbreviation\n");
        logfput(msgbuf);
        strcpy(msgbuf, "FROM(F)=volume:group,user\n");
        logfput(msgbuf);
        strcpy(msgbuf, "OPT(O)=1/2/3\n");
        logfput(msgbuf);
        strcpy(msgbuf, "ID(I)=userid,password\n");
        logfput(msgbuf);
        strcpy(msgbuf, "[MSG(M)=msg log file name]\n");
        logfput(msgbuf);
        break;
    default:
        strcpy(msgbuf, "Error  Other Error\n");
        logfput(msgbuf);
        break;
    } /* end of switch */
    return(0);
}
/************************************************************************
*                                                                       *
*   Output error code for MC_usrid                                      *
*                                                                       *
*       errcode : Error Code                                            *
*                                                                       *
************************************************************************/
short logusrid(long errcode)
{
    char    msgbuf[MSG_STRING_MAX];

    /*--------------*/
    /* Initial      */
    /*--------------*/
    memset(msgbuf, '\0', sizeof(msgbuf));
    /*--------------------------------------*/
    /* Output the drawing name and message  */
    /*--------------------------------------*/
    switch(errcode) {
    case 0:
        strcpy(msgbuf, "Complete\n");
        break;
    case 5:
        strcpy(msgbuf, "Invalid User ID Specified");
        break;
    case 6:
        strcpy(msgbuf, "Invalid Password Specified");
        break;
    default:
        strcpy(msgbuf, "Unexpected Error");
        break;
    } /* end of switch */
    logfput(msgbuf);
    return(0);
}
