/************************************************************************
*                                                                       *
*   The process of deleting unused detail                               *
*                                                                       *
*       opt     :  Option (1:Unused, 2:Unused & No show,                *
*                          3:Unused Detail of other than nested)        *
*       vol     :  Volume                                               *
*       group   :  Group                                                *
*       user    :  User                                                 *
*                                                                       *
************************************************************************/
#include    <stdlib.h>
#include    <malloc.h>
#include    <memory.h>
#define     VOL_4                       /* Drive is 4 characters */
#include    "accfnc.h"

extern short loghed(char*,char*,char*);
extern short logdwgid(char*,long);
static short deldtl_dwg(long,long*);
static short deldtl_unused(long);
static short deldtl_not_nested(long);
static short deldit_noshow_all(long);
static void  deldit_noshow_all_current(long);
static short deldit_noshow_part(long,long,long*);
static void  deldit_noshow_part_current(long,long,long*);
static short check_ditto_current(long);
static short dtlpack_dwg(long*);

#define SPACE_CHAR            ' '
#define WILD_CHAR             '?'

#define DWGID_LEN              20

#define PERMISSION_RW           1
#define PERMISSION_RO           2
#define PERMISSION_PW           3
#define PERMISSION_LK           4
#define PERMISSION_SC           5

#define IDATA_OFFSET_PERMISSION 2
#define IDATA_OFFSET_OVERLAY    3
#define IDATA_OFFSET_3DELM      4

#define MODEL_NO                2
#define TEMP_MODEL_NO           3

#define FALSE                   0
#define TRUE                    1

short dtlpack(long opt, char *vol, char *group, char *user)
{
    short   s_retc, retc;
    long    ii, jj;
    long    model_count;            /* Number of Models */
    long    *idata;                 /* Work Area */
    long    idummy;                 /* Dummy */
    long    compmod;
    long    user_count;
    long    del_flag;
    long    pack_flag;
    char    dwgid[DWGID_LEN+1];     /* Drawing Name */
    char    wild[DWGID_LEN+1];      /* Wild Card */
    char    *id;                    /* Work Area */
    char    dummy;                  /* Dummy */

    /*--------------------------------------*/
    /* Output the header message            */
    /*--------------------------------------*/
    loghed(vol, group, user);
    user_count = 1L;
    retc = 0;
    memset(wild, WILD_CHAR, sizeof(wild));
    for (ii = 0L; ii < user_count; ii++) {
        /*--------------------------------------*/
        /* Query the number of models           */
        /*--------------------------------------*/
        model_count = 0L;
        s_retc = MC_drwlst(1L, vol, group, user, WILD_CHAR, wild, 0L,
                        &model_count, &dummy, &idummy, &dummy);
        if (s_retc != 0) {
            if (s_retc == 13 ||
                s_retc == 18) {
                /* Model is not found */
                continue;
            }
            else {
                memset(dwgid, '\0', sizeof(dwgid));
                memset(dwgid, SPACE_CHAR, DWGID_LEN);
                logdwgid(dwgid, (long)s_retc);
                continue;
            } 
        }
        /*--------------------------*/
        /* Allocate the area        */
        /*--------------------------*/
        id = (char *)malloc(DWGID_LEN * model_count);
        if (id == NULL) {
            retc = 200;
            break;
        }
        idata = (long *)malloc(sizeof(long) * DWGID_LEN * model_count);
        if (idata == NULL) {
            free(id);
            retc = 200;
            break;
        }
        /*----------------------------------*/
        /* Query the drawing information    */
        /*----------------------------------*/
        s_retc = MC_drwlst(2L, vol, group, user, WILD_CHAR, wild, 0L,
                            &model_count, id, idata, &dummy);
        if (s_retc != 0L) {
            memset(dwgid, '\0', sizeof(dwgid));
            memset(dwgid, SPACE_CHAR, DWGID_LEN);
            logdwgid(dwgid, (long)s_retc);
            free(id);
            free(idata);
            continue;
        }
        /*------------------*/
        /* Set Drive        */
        /*------------------*/
        MC_budriv(1L, 2L, vol, 10L);
        for (jj = 0L; jj < model_count; jj++) {
            /*--------------------------*/
            /* Copy the drawing name    */
            /*--------------------------*/
            memset(dwgid, '\0', sizeof(dwgid));
            memcpy(dwgid, &id[20*jj], DWGID_LEN);
            /*------------------------------*/
            /* Check the model permission   */
            /*------------------------------*/
            if ( idata[20*jj+IDATA_OFFSET_PERMISSION] != PERMISSION_RW
              && idata[20*jj+IDATA_OFFSET_PERMISSION] != PERMISSION_SC
               )
            {
                switch(idata[20*jj+IDATA_OFFSET_PERMISSION]) {
                case PERMISSION_RO:
                    logdwgid(dwgid, 20L);
                    break;
                case PERMISSION_PW:
                    logdwgid(dwgid, 23L);
                    break;
                case PERMISSION_LK:
                    logdwgid(dwgid, 1002L);
                    break;
                default:
                    break;
                } /* end of switch */
                continue;
            }
            /*-------------------------------------------*/
            /* Check the parent drawing of overlay model */
            /*-------------------------------------------*/
            if (idata[20*jj+IDATA_OFFSET_OVERLAY] != 1L) {
                logdwgid(dwgid, 0L);
                continue;
            }
            /*--------------------------------------*/
            /* Check the drawing including 3D model */
            /*--------------------------------------*/
            if (idata[20*jj+IDATA_OFFSET_3DELM] != 1L) {
                logdwgid(dwgid, 21L);
                continue;
            }
            /*--------------------------*/
            /* Open the model           */
            /*--------------------------*/
            s_retc = MC_buopen(2L, MODEL_NO, group, user, dwgid, 10L);
            if (s_retc != 0) {
                logdwgid(dwgid, (long)s_retc);
                continue;
            }

            /*---------------------------------------*/
            /* The process of deleting unused detail */
            /*---------------------------------------*/
            s_retc = deldtl_dwg(opt, &del_flag);
            if (s_retc != 0) {
                logdwgid(dwgid, (long)s_retc);
                MC_mdlclos(MODEL_NO);
                continue;
            }
            /*--------------------------------------*/
            /* The process packs the drawings       */
            /*--------------------------------------*/
            s_retc = dtlpack_dwg(&pack_flag);
            if (s_retc != 0) {
                logdwgid(dwgid, (long)s_retc);
                MC_mdlclos(MODEL_NO);
                continue;
            }
            if (!del_flag && !pack_flag) {
                logdwgid(dwgid, 1000L);
                MC_mdlclos(MODEL_NO);
                continue;
            }

            /*----------------------------------------------*/
            /* Set the compress mode and file the drawing   */
            /*----------------------------------------------*/
            MC_flcompst(vol, group, user, dwgid, &compmod);
            MC_flcomp(1L, &compmod);
            s_retc = MC_bufile(5L, MODEL_NO, group, user, dwgid, 10L);

            logdwgid(dwgid, (long)s_retc);
            /*----------------------*/
            /* Close the model      */
            /*----------------------*/
            MC_mdlclos(MODEL_NO);
        } /* end of for */
        /*--------------*/
        /* Free         */
        /*--------------*/
        free(id);
        free(idata);
    } /* end of for */
    return(0);
}
/************************************************************************
*                                                                       *
*   Delet unused detail                                                 *
*                                                                       *
*       opt         :  Option (1:Unused, 2:Unused & No show,            *
*                          3:Unused Detail of other than nested)        *
*       del_flag    :  0:Not deleted 1:Deleted                          *
*                                                                       *
************************************************************************/
static short deldtl_dwg(long opt, long *del_flag)
{
    short   retc;
    long    save_ivdtln;
    long    new_ivdtln;
    long    *new_ivdtar;

    *del_flag = FALSE;

    /* Get the number of Detail */
    MC_gvdnum(2L, MODEL_NO, &save_ivdtln);
    if (save_ivdtln == 0L) return(0);

    /*-----------------*/
    /* Check the mode  */
    /*-----------------*/
    switch(opt) {
    case 1:     /* Unused */
        /* Delete unused Detail */
        retc = deldtl_unused(MODEL_NO);
        break;
    case 2:     /* Unused & No show */
        /* Check no show Detail */
        MC_mdlcopy(2L, MODEL_NO, TEMP_MODEL_NO, "DTLPACK TEMP_MODEL  ");
        retc = deldit_noshow_all(TEMP_MODEL_NO);
        retc = deldtl_unused(TEMP_MODEL_NO);

        /* Get the number of Detail */
        MC_gvdnum(2L, TEMP_MODEL_NO, &new_ivdtln);
        if (save_ivdtln == new_ivdtln) break;
        new_ivdtar = malloc(sizeof(long) * new_ivdtln);
        if (new_ivdtar == NULL) return(200);

        /* Delete no-show Ditto */
        MC_budrct(3L, TEMP_MODEL_NO, &new_ivdtln, new_ivdtar, 10L);
        retc = deldit_noshow_part(MODEL_NO, new_ivdtln, new_ivdtar);
        /* Delete unused Detail */
        retc = deldtl_unused(MODEL_NO);
        free(new_ivdtar);
        break;
    case 3:     /* Unused Detail of other than nested */
        retc = deldtl_not_nested(MODEL_NO);
        break;
    } /* end of switch */
    /* Get the number of Detail */
    MC_gvdnum(2L, MODEL_NO, &new_ivdtln);
    if (save_ivdtln != new_ivdtln) *del_flag = TRUE;

    return(retc);
}

/************************************************************************
*                                                                       *
*   Delete unused Detail                                                *
*                                                                       *
*       modno       :  No. of Model                                     *
*                                                                       *
************************************************************************/
static short deldtl_unused(long modno)
{
    short   retc;
    long    ii;
    long    ivdtln;
    long    *ivdtar;
    long    delflag;
    char    detnam_dum[8] = "";

    delflag = TRUE;

    /* Get the number of Detail */
    MC_gvdnum(2L, modno, &ivdtln);
    if (ivdtln == 0L) return(0);
    ivdtar = malloc(sizeof(long) * ivdtln);
    if (ivdtar == NULL) return(200);

    while ( delflag ) {
        delflag = FALSE;

        /* Get the number of Detail */
        MC_gvdnum(2L, modno, &ivdtln);
        if (ivdtln == 0L) break;
        /* Create a Detail number list */
        MC_budrct(3L, modno, &ivdtln, ivdtar, 10L);

        /* Delete Detail */
        for ( ii=0; ii<ivdtln; ii++ ) {
            retc = MC_bubdtl(2L, modno, detnam_dum, &ivdtar[ii], 10L);
            if (retc == 9) continue;
            delflag = TRUE;
        }
    }   /* end of while */
    free(ivdtar);
    return(0);
}
/************************************************************************
*                                                                       *
*   Delete no show Ditto                                                *
*                                                                       *
*       modno       :  No. of Model                                     *
*                                                                       *
************************************************************************/
static short deldit_noshow_all(long modno)
{
    long    ii;
    long    ivdtln;
    long    *ivdtar;
    long    ivudet;
    long    number;

    /* Get the number of View */
    MC_gvdnum(1L, modno, &ivdtln);

    /* Allocate the area */
    ivdtar = malloc(sizeof(long) * ivdtln);
    if (ivdtar == NULL) return(200);

    /* Get View number list */
    MC_budrct(1L, modno, &ivdtln, ivdtar, 10L);

    /* Delete no show Ditto of all view */
    for (ii=0; ii<ivdtln; ii++) {
        ivudet = ivdtar[ii];
        MC_bucds(1L, modno, 1L, &ivudet, &number, 10L);

        /* Delete no show Ditto of the current view */
        deldit_noshow_all_current(modno);
    }
    free(ivdtar);

    /* Get the number of Detail */
    MC_gvdnum(2L, modno, &ivdtln);
    /* No Detail */
    if ( ivdtln == 0L ) return ( 0L );

    /* Allocate the area */
    ivdtar = malloc(sizeof(long) * ivdtln);
    if (ivdtar == NULL) return(200);

    /* Get Detail number list */
    MC_budrct(3L, modno, &ivdtln, ivdtar, 10L);

    /* Delete no show Ditto of all detail */
    for (ii=0; ii<ivdtln; ii++) {
        ivudet = ivdtar[ii];
        MC_bucds(1L, modno, 1L, &ivudet, &number, 10L);

        /* Delete no show Ditto of the current detail */
        deldit_noshow_all_current(modno);
    }
    free(ivdtar);

    return(0);
}
/***********************************************************************
*                                                                      *
*   Delete no show Ditto of the current view                           *
*                                                                      *
*           modno  :  Model No                                         *
*                                                                      *
***********************************************************************/
static void deldit_noshow_all_current (long modno)
{
    short   bunext_ret;
    long    nxtptr;
    long    itype;
    long    save_paptr;
    long    mptr[2];
    long    bisho;


    mptr[0] = 1L;
    for ( ; ; ) {
        bunext_ret = MC_bunext(1L, modno, mptr[0], &nxtptr, 10L);

        itype = MC_biityp(modno, mptr[0]);
        switch ( itype ) {
        /* Ditto */
        case 1050:
            bisho = MC_bisho(modno, mptr[0]);
            if (bisho == 0L)
                MC_budel(1L, modno, mptr, 10L);
            break;

        /* Parts */
        case 1700:
        case 1750:
            MC_pmode(2L, modno, &save_paptr);
            MC_pmode(1L, modno, mptr);

            /* Delete no show ditto in parts */
            deldit_noshow_all_current ( modno );

            if (save_paptr == 0L)
                MC_pmode(0L, modno, &save_paptr);
            else
                MC_pmode(1L, modno, &save_paptr);

        default:
            break;
        } /* end of switch */

        if (bunext_ret != 0) break;
        mptr[0] = nxtptr;
        continue;
    }

    return;
}
/************************************************************************
*                                                                       *
*   Delete other than the specified Detail                              *
*                                                                       *
*       modno   :  No. of Model                                         *
*       idtln   :  Number of Details to leave                           *
*       idtar   :  Detail list to leave                                 *
*                                                                       *
************************************************************************/
static short deldit_noshow_part(long modno, long idtln, long *idtar)
{
    long    ii;
    long    ivdtln;
    long    *ivdtar;
    long    ivudet;
    long    number;


    /* Get the number of View */
    MC_gvdnum(1L, modno, &ivdtln);

    /* Allocate the area */
    ivdtar = malloc(sizeof(long) * ivdtln);
    if (ivdtar == NULL) return(200);

    /* Get View number list */
    MC_budrct(1L, modno, &ivdtln, ivdtar, 10L);

    /* Delete no show Ditto of all view */
    for (ii=0; ii<ivdtln; ii++) {
        ivudet = ivdtar[ii];
        MC_bucds(1L, modno, 1L, &ivudet, &number, 10L);

        /* Delete no show Ditto of the current view */
        deldit_noshow_part_current(modno, idtln, idtar);
    }
    free(ivdtar);


    /* Get the number of Detail */
    MC_gvdnum(2L, modno, &ivdtln);
    /* No Detail */
    if ( ivdtln == 0L ) return ( 0L );

    /* Allocate the area */
    ivdtar = malloc(sizeof(long) * ivdtln);
    if (ivdtar == NULL) return(200);

    /* Get Detail number list */
    MC_budrct(3L, modno, &ivdtln, ivdtar, 10L);

    /* Delete no show Ditto of all detail */
    for (ii=0; ii<ivdtln; ii++) {
        ivudet = ivdtar[ii];
        MC_bucds(1L, modno, 1L, &ivudet, &number, 10L);

        /* Delete no show Ditto of the current detail */
        deldit_noshow_part_current(modno, idtln, idtar);
    }
    free(ivdtar);


    return(0);
}
/***********************************************************************
*                                                                      *
*   Delete no show Ditto of the current view                           *
*                                                                      *
*       modno   :  No. of Model                                        *
*       idtln   :  Number of Details to leave                          *
*       idtar   :  Detail list to leave                                *
*                                                                      *
***********************************************************************/
static void deldit_noshow_part_current (long modno, long idtln, long *idtar)
{
    short   bunext_ret;
    long    nxtptr;
    long    itype;
    long    save_paptr;
    long    mptr[2];
    long    bisho;
    long    ii;
    long    idtlno;
    long    ifill;
    long    imirr;
    double  xpvt;
    double  ypvt;
    double  ang;
    double  scale;


    mptr[0] = 1L;
    for ( ; ; ) {
        bunext_ret = MC_bunext(1L, modno, mptr[0], &nxtptr, 10L);

        itype = MC_biityp(modno, mptr[0]);
        switch ( itype ) {
        /* Ditto */
        case 1050:
            MC_gtdit(modno, mptr[0], &idtlno, &xpvt, &ypvt, &ang, &scale, &ifill, &imirr);
            for ( ii=0; ii<idtln; ii++ ) {
                if (idtlno == idtar[ii]) break;
            }
            if (ii < idtln)  break;
            bisho = MC_bisho(modno, mptr[0]);
            if (bisho == 0L)
                MC_budel(1L, modno, mptr, 10L);
            break;

        /* Parts */
        case 1700:
        case 1750:
            MC_pmode(2L, modno, &save_paptr);
            MC_pmode(1L, modno, mptr);

            /* Delete no show ditto in parts */
            deldit_noshow_part_current(modno, idtln, idtar);

            if (save_paptr == 0L)
                MC_pmode(0L, modno, &save_paptr);
            else
                MC_pmode(1L, modno, &save_paptr);

        default:
            break;
        } /* end of switch */

        if (bunext_ret != 0) break;
        mptr[0] = nxtptr;
        continue;
    }

    return;
}
/************************************************************************
*                                                                       *
*   Delete nested Detail                                                *
*                                                                       *
*       modno       :  No. of Model                                     *
*                                                                       *
************************************************************************/
static short deldtl_not_nested(long modno)
{
    short   retc;
    long    ii;
    long    ivdtln;
    long    *ivdtar;
    long    ivudet;
    long    number;
    char    detnam_dum[8] = "";


    /* Get the number of Detail */
    MC_gvdnum(2L, modno, &ivdtln);
    if (ivdtln == 0L) return(0);

    ivdtar = malloc(sizeof(long) * ivdtln);
    if (ivdtar == NULL) return(200);


    /* Get Detail number list */
    MC_budrct(3L, modno, &ivdtln, ivdtar, 10L);


    for ( ii=0; ii<ivdtln; ii++ ) {
        ivudet = ivdtar[ii];
        MC_bucds(1L, modno, 1L, &ivudet, &number, 10L);

        retc = check_ditto_current (modno);
        if (retc == TRUE) continue;
        
        /* Delete Detail */
        retc = MC_bubdtl(2L, modno, detnam_dum, &ivdtar[ii], 10L);
    }

    free(ivdtar);

    return(0);
}
/***********************************************************************
*                                                                      *
*       Check a ditto of the current view                              *
*                                                                      *
*           modno  :  Model No                                         *
*                                                                      *
***********************************************************************/
static short check_ditto_current (long modno)
{
    long    nxtptr;
    long    itype;
    long    save_paptr;
    long    mptr[2];
    short   bunext_ret;
    short   retc;


    mptr[0] = 1L;
    for ( ; ; ) {
        bunext_ret = MC_bunext(1L, modno, mptr[0], &nxtptr, 10L);

        itype = MC_biityp(modno, mptr[0]);
        switch ( itype ) {
        /* Ditto */
        case 1050:
            return(TRUE);
            break;

        /* Parts */
        case 1700:
        case 1750:
            MC_pmode(2L, modno, &save_paptr);
            MC_pmode(1L, modno, mptr);

            retc = check_ditto_current ( modno );

            if (save_paptr == 0L)
                MC_pmode(0L, modno, &save_paptr);
            else
                MC_pmode(1L, modno, &save_paptr);

            if ( retc == TRUE ) return(retc);

        default:
            break;
        } /* end of switch */

        if (bunext_ret != 0) break;
        mptr[0] = nxtptr;
        continue;
    }

    return(FALSE);
}
/*-----------------------------------------------------*/
/* Detail pack                                         */
/*      long pack_flag : TRUE  : Packed                */
/*                       FALSE : Not Packed            */
/*-----------------------------------------------------*/
static short dtlpack_dwg(long *pack_flag)
{
    long    save_ivdtln;
    long    new_ivdtln;
    long    *save_ivdtar = NULL;
    long    *new_ivdtar = NULL;

    *pack_flag = FALSE;

    /* Get the number of Detail */
    MC_gvdnum( 2L, MODEL_NO, &save_ivdtln );
    if (save_ivdtln == 0L) return(0);

    save_ivdtar = malloc(sizeof(long) * save_ivdtln);
    if (save_ivdtar == NULL) return(200);

    /* Get Detail number list */
    MC_budrct(3L, MODEL_NO, &save_ivdtln, save_ivdtar, 10L);

    /* Detail Packed */
    MC_bupack(2L, MODEL_NO, 10L);
    MC_bupack(1L, MODEL_NO, 10L);

    /* Get the number of Detail */
    MC_gvdnum(2L, MODEL_NO, &new_ivdtln);
    new_ivdtar = malloc(sizeof(long) * new_ivdtln);
    if ( new_ivdtar == NULL ) {
        free(save_ivdtar);
        return(200);
    }

    /* Get Detail number list */
    MC_budrct(3L, MODEL_NO, &new_ivdtln, new_ivdtar, 10L);

    /* Comparison of Detail numbers */
    if ( save_ivdtln != new_ivdtln ) {
        *pack_flag = TRUE;
    }
    else {
        /* Check the last detail number */
        if (save_ivdtar[save_ivdtln-1] != new_ivdtar[new_ivdtln-1]) {
            *pack_flag = TRUE;
        }
    }

    free( save_ivdtar );
    free( new_ivdtar );

    return(0);
}
